from gai.common.http_utils import http_post
import gai.common.ConfigHelper as ConfigHelper
API_BASEURL = ConfigHelper.get_api_baseurl()

class TTSClient:

    def __call__(self, input, generator="xtts-2", stream=True,**generator_params):
        if not input:
            raise Exception("The parameter 'input' is required.")
        data = {
            "model": generator,
            "input": input,
            "stream": stream,
            **generator_params
        }
        response = http_post(f"{API_BASEURL}/gen/v1/audio/speech",data)
        return response




from gai.gen.TTTClient import TTTClient
from gai.gen.STTClient import STTClient
from gai.gen.TTSClient import TTSClient
from gai.gen.ITTClient import ITTClient
from gai.gen.RAGClient import RAGClient

class GaigenClient:

    def __call__(self, category, **model_params):
        if category.lower() == "ttt":
            ttt = TTTClient()
            return ttt(**model_params)
        elif category.lower() == "stt":
            stt = STTClient()
            return stt(**model_params)
        elif category.lower() == "tts":
            tts = TTSClient()
            return tts(**model_params)
        elif category.lower() == "itt":
            itt = ITTClient()
            return itt(**model_params)
        elif category.lower() == "index":
            rag = RAGClient()
            return rag.index_file(**model_params)
        elif category.lower() == "retrieve":
            rag = RAGClient()
            return rag.retrieve(**model_params)
        else:
            raise Exception(f"Unknown category: {category}")



from gai.gen.ChunkWrapper import ChunkWrapper 
from gai.gen.OpenAIChunkWrapper import OpenAIChunkWrapper
from gai.gen.AnthropicChunkWrapper import AnthropicChunkWrapper
import gai.common.ConfigHelper as ConfigHelper
from gai.common.http_utils import http_post
from gai.common.utils import chat_string_to_list, chat_list_to_string
import json

API_BASEURL = ConfigHelper.get_api_baseurl()

class TTTClient:

    def api(self, generator=None, messages=None, stream=True, **generator_params):
        if not messages:
            raise Exception("Messages not provided")
        
        if isinstance(messages,str):
            messages = chat_string_to_list(messages)

        cli_config = ConfigHelper.get_cli_config()
        if not generator:
            generator = cli_config["default_generator"]

        data = {
            "model": generator,
            "messages":messages,
            "stream":stream,
            **cli_config["generators"][generator]["default"],
            **generator_params
            }

        def streamer(response):
            for chunk in response.iter_lines():
                yield ChunkWrapper(chunk)

        response = http_post(f"{API_BASEURL}/gen/v1/chat/completions",data)
        if not stream:
            response.decode = lambda: response.json()["choices"][0]["message"]["content"]
            return response
        return streamer(response)        


    def __call__(self, generator=None, messages=None, stream=True, **generator_params):
        if generator=="gpt-4":
            return self.gpt_4(messages=messages,stream=stream,**generator_params)
        elif generator=="claude2-100k":
            return self.claude_2(messages=messages,stream=stream,**generator_params)
        return self.api(generator,messages,stream,**generator_params)


    def gpt_4(self, messages=None, stream=True, **generator_params):
        import os, openai
        from openai import OpenAI
        from dotenv import load_dotenv        
        load_dotenv()
        if not os.environ.get("OPENAI_API_KEY"):
            raise Exception("OPENAI_API_KEY not found in environment variables")
        openai.api_key = os.environ["OPENAI_API_KEY"]
        client = OpenAI()

        if not messages:
            raise Exception("Messages not provided")
       
        def streamer(response):
            for chunk in response:
                yield OpenAIChunkWrapper(chunk)

        model="gpt-4"
        if isinstance(messages,str):
            messages = chat_string_to_list(messages)
        response = client.chat.completions.create(
                        model=model,
                        messages=messages,
                        stream=stream,
                        **generator_params
                    )

        if not stream:
            response.decode = lambda: response.choices[0].message.content
            return response
        return streamer(response) 
    
    def claude_2(self, messages=None, stream=True, **generator_params):
        import os
        from anthropic import Anthropic
        from dotenv import load_dotenv        
        load_dotenv()
        if not os.environ.get("ANTHROPIC_API_KEY"):
            raise Exception("ANTHROPIC_API_KEY not found in environment variables")
        client = Anthropic()

        if not messages:
            raise Exception("Messages not provided")
       
        def streamer(response):
            for chunk in response:
                yield AnthropicChunkWrapper(chunk)

        model="claude-2"
        message=messages
        if isinstance(messages,list):
            message = chat_list_to_string(messages)
        prompt_template = "\n\nHuman: {message}\n\nAssistant:"
        messages = prompt_template.format(message=message)
        response = client.completions.create(
                        model=model,
                        prompt=messages,
                        stream=stream,
                        **generator_params
                    )

        if not stream:
            response.decode = lambda: response.completion
            return response
        return streamer(response)     


from gai.gen.ChunkWrapper import ChunkWrapper
import gai.common.ConfigHelper as ConfigHelper
from gai.common.http_utils import http_post
from gai.common.image_utils import base64_to_imageurl
API_BASEURL = ConfigHelper.get_api_baseurl()

class ITTClient:

    def __call__(self, model="llava-transformers", base64_image=None, message_text=None, stream=True, **model_params):
        if not base64_image:
            raise Exception("The parameter 'base64_imageurl' is required.")
        if not message_text:
            message_text="Describe this image."

        base64_imageurl = base64_to_imageurl(base64_image)
        data = {
            "model": model,
            "messages": [
                {"role":"user",
                "content":[
                    {"type":"text","text":message_text},
                    {"type":"image_url","image_url":base64_imageurl}
                ]}
            ],
            "stream": stream,
        }

        def streamer(response):
            for chunk in response.iter_lines():
                yield ChunkWrapper(chunk)

        response = http_post(f"{API_BASEURL}/gen/v1/vision/completions",data)
        if not stream:
            response.decode = lambda: response.json()["choices"][0]["message"]["content"]
            return response
        return streamer(response)
    



import gai.common.ConfigHelper as ConfigHelper
from gai.common.http_utils import http_post
API_BASEURL = ConfigHelper.get_api_baseurl()

class STTClient:

    def __call__(self, generator=None, file=None):
        if not file:
            raise Exception("No file provided")
        
        files = {
            "model": (None,generator),
            "file": (file.name, file.read())
        }

        response = http_post(f"{API_BASEURL}/gen/v1/audio/transcriptions",files=files)
        response.decode = lambda: response.json()["text"]
        return response


import subprocess

def play_audio(response):
    ffplay_cmd = ["ffplay", "-nodisp", "-autoexit", "-"]
    ffplay_proc = subprocess.Popen(ffplay_cmd, stdin=subprocess.PIPE, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
    for chunk in response.raw.stream(1024, decode_content=False):
        if chunk:
            ffplay_proc.stdin.write(chunk)

    # close on finish
    ffplay_proc.stdin.close()
    ffplay_proc.wait()


import json
class ChunkWrapper:

    # chunk is json in binary representation
    def __init__(self, chunk):
        self.chunk = chunk

    def __str__(self):
        return self.chunk.decode('utf-8')

    def decode(self):
        decoded_chunk = json.loads(self.chunk.decode('utf-8'))
        return decoded_chunk["choices"][0]["delta"]["content"]
    
    def __dict__(self):
        return json.loads(self.chunk.decode('utf-8'))

